@@ -4,7 +4,11 @@
 # Taken
 # grok sense gist pygist fathom
 
-import types, re
+import sys, types, re
+
+# Python 2.4 doesn't have functools
+if sys.version_info >= (2,5):
+    import functools
 
 # Handle numpy types if numpy is available.
 try: import numpy
@@ -15,7 +19,7 @@ except ImportError: numpy = False
 # installed but you're running under standard python, you get a name
 # error (from get_ipython() function failing to resolve)
 try: import magic
-except ImportError: pass
+except (ImportError, SyntaxError): pass
 
 # This is just to test for a poorly written __cmp__ function in
 # distutils.version.LooseVersion.  Importing distutils should always work but I don't want to give the impression that we need it if something goes wrong.
@@ -47,7 +51,9 @@ verbose = False
 
 apropos_dict_types = [types.DictType]
 apropos_list_types = [types.ListType, types.TupleType]
-apropos_instance_types = [types.InstanceType, types.ModuleType]
+apropos_instance_types = [types.ModuleType]
+if sys.version_info < (3,):
+    apropos_instance_types += [types.InstanceType]
 
 ##################################################
 # Information about types for recursive_types() function.
@@ -79,7 +85,24 @@ class sstr(object):
 
 def every(args): 
     """Return True if all elements of args are True."""
-    return reduce(lambda x,y: x and y, args, True)
+    # Python 2.5 has the functools module, but reduce isn't in it.
+    if sys.version_info >= (2,6):
+        return functools.reduce(lambda x,y: x and y, args, True)
+    else:
+        return reduce(lambda x,y: x and y, args, True)
+
+def isstring(obj):
+    """Test if an object is a string for different python versions."""
+    # Early Python only had one string type
+    # if type(obj) is str
+    # Middle-aged Python had several:
+    # if type(obj) in types.StringTypes
+    # Modern python has one again
+    # if type(obj) is str
+    if sys.version_info < (3,):
+        return type(obj) in types.StringTypes
+    else:
+        return type(obj) is str
 
 ##################################################
 ## Introspection
@@ -112,7 +135,7 @@ def gist(obj, verbose=False, pretty=True):
             except: attr = Exception
             info.append((name, type(attr)))
 
-    types = sorted(set([el[1] for el in info]))
+    types = sorted(set([el[1] for el in info]), key=str)
     # TODO result used to be a list, not a dict.  Do I prefer that
     # since I'll have deterministic ordering in printouts?  Might make
     # programmatic processing of output worse, but I don't do that
@@ -283,7 +306,7 @@ def search_equal(needle, name, obj):
 
         try: 
             result = numpy.all(needle==obj)
-        except Exception as e:
+        except Exception:
             if verbose: 
                 print "Exception encountered in test for equality, assuming unequal..."
             result = False
@@ -293,7 +316,7 @@ def search_equal(needle, name, obj):
     # evidently not equal.
     try: 
         result = (needle == obj)
-    except Exception as e:
+    except Exception:
         if verbose: 
             print "Exception encountered in test for equality, assuming unequal..."
         result = False
@@ -301,11 +324,11 @@ def search_equal(needle, name, obj):
 
 def search_doc(needle, name, obj):
     """Match if needle is contained in the docstring of obj"""
+    # Some functions have __doc__ attributes that appear to be
+    # functions... Only check ones that are strings
     return (hasattr(obj, '__doc__') and 
-            # Some functions have __doc__ attributes that appear to be
-            # functions... Only check ones that are strings
-            type(obj.__doc__) in types.StringTypes
-            and needle in obj.__doc__)
+              isstring(obj.__doc__)
+              and needle in obj.__doc__)
     
 def search_name_regexp(needle, name, obj):
     """Match if regexp needle matches name"""
@@ -322,7 +345,7 @@ def search_doc_regexp(needle, name, obj):
     return (hasattr(obj, '__doc__') 
             # Some functions have __doc__ attributes that appear to be
             # functions... Only check ones that are strings        
-            and type(obj.__doc__) in types.StringTypes 
+            and isstring(obj.__doc__)
             and re.search(needle, obj.__doc__))
 
 ##############################
@@ -492,7 +515,7 @@ class DictIntrospector (Introspector):
         # return tuple of obj, name, access_name
         k = self.iter.next()
         # TODO -- completely skip non-string key entries
-        while type(k) is not types.StringType \
+        while not isstring(k) \
               or (self.exclude and k.startswith(self.exclude)):
             k = self.iter.next()
         return self.dict[k], k, '[' + k + ']'
