@@ -8,7 +8,7 @@ if not hasattr(unittest, 'skipIf'):
             """Tests require either the Python 2.7 or later version of unittest or
             the unittest2 module."""
 
-import tempfile
+import sys, tempfile
 import grasp
 from grasp import *
 
@@ -43,23 +43,44 @@ class IntrospectionTest(unittest.TestCase):
         # Python
         
         self.excludes = [types.BufferType, types.CodeType,
-                         types.DictProxyType, types.EllipsisType,
+                         types.EllipsisType,
                          types.FrameType, types.GeneratorType,
-                         types.GetSetDescriptorType,
-                         types.MemberDescriptorType,
                          types.NotImplementedType,
-                         types.TracebackType,
-                         # Not sure why this isn't showing up under exceptions
+                         types.TracebackType ]
 
-                         GeneratorExit,
-                         # TODO -- do these belong here?
-                         bytearray, memoryview]
+        # Added in Python 3.3
+        if sys.version_info >= (3,3):
+            self.excludes += [types.SimpleNamespace,
+                              types.MappingProxyType,
+                              __loader__]
+
+        # Added in Python 3
+        if sys.version_info > (3,):
+            self.excludes += [bytes, filter, slice, zip, map]
+
+        # Removed in Python 3
+        if sys.version_info < (3,):
+            self.excludes += [types.DictProxyType]
+
+        # Added in Python 2.7
+        if sys.version_info >= (2,7):
+            self.excludes += [memoryview]
+
+        # Added in Python 2.6
+        if sys.version_info >= (2,6):
+            self.excludes += [bytearray]
+
+        # Added in Python 2.5
+        if sys.version_info >= (2,5):
+            self.excludes += [ types.GetSetDescriptorType,
+                               types.MemberDescriptorType, 
+                               # Not sure why this doesn't show up under exceptions
+                               GeneratorExit]
 
         # These are the ones in types module
         self.objs = [True, # Boolean,
                      repr, # built-in function
                      1j, # Complex
-                     types.ClassType('foo', (object,), {}), # Class
                      dict(a=1), # Dictionary
                      self.tf,    # File
                      1.1,  # Float
@@ -75,7 +96,6 @@ class IntrospectionTest(unittest.TestCase):
                      unittest, # Module
                      None, # None
                      object(), # Object
-                     types.SliceType(1),  # Slice
                      'blah', # String
                      (1,2),  # Tuple
                      int, # type
@@ -86,6 +106,12 @@ class IntrospectionTest(unittest.TestCase):
                      xrange(3), # xrange
                      ]
 
+        if sys.version_info < (3,):
+            self.objs += [types.ClassType('foo', (object,), {}), # Class
+                          types.SliceType(1),  # Slice
+            ] 
+
+        
         # These I found in __builtins__
         self.objs += [super(test_new_obj),  # Super
                       staticmethod(func), # staticmethod                
@@ -98,9 +124,12 @@ class IntrospectionTest(unittest.TestCase):
                       Exception(),  # Exception                 
                       KeyboardInterrupt(),  
                       SystemExit(),  
-                      BaseException(),  
                       ]
 
+        if sys.version_info >= (2,5):
+            self.objs += [ BaseException() ]
+        
+
         # These I found in __builtins__ but can't be instantiated
         self.excludes += [basestring]
 
@@ -152,19 +181,23 @@ class IntrospectionTest(unittest.TestCase):
     def test_type_coverage(self):        
         # Check to see if any types aren't covered        
         covered_types = self.excludes + [type(obj) for obj in self.objs]
-        els = __builtins__.values() \
-              + [getattr(types, name) for name in dir(types)] 
-        all_types = [el for el in els if type(el) is type]
 
-        is_covered = [t in covered_types
-                     # don't bother explicitly listing all exceptions
-                     or issubclass(t, Exception)
-                     # TODO -- something weird is going on with class types
-                     or t is types.ClassType
-                     # TODO -- strange -- reversed is in the list of objects
-                     or t is reversed
-                     for t in all_types]
-        self.assertTrue(every(is_covered))
+        # Keep names of objects to make finding new types easier.
+        names_and_vals = __builtins__.items() \
+                         + [(name, getattr(types, name)) for name in dir(types)] 
+        all_types = [(name, val) for name, val in names_and_vals if type(val) is type]
+        
+        uncovered = [(name, val, type(val))
+                      for name, val in all_types
+                      if not (type(val) in covered_types
+                              # don't bother explicitly listing all exceptions
+                              or issubclass(type(val), Exception)
+                              # TODO -- something weird is going on with class types
+                              or type(val) is types.ClassType
+                              # TODO -- strange -- reversed is in the list of objects
+                              or type(val) is reversed)]
+
+        self.assertEqual(len(uncovered), 0)
 
     def test_gist(self):
         for el in self.objs:
